/*
 * Copyright (c) 2013-2023 NIBIO.
 *
 * This file is part of VIPSWeb 
 * (see https://gitlab.nibio.no/VIPS/VIPSWeb).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */



/**
 * Application logic for the roughage nutrition calculation form
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */

/**
 * Runs the nutrition model asyncronously. Upon successful running it runs the grass drying model if requested
 */
function runNutritionModel(formId)
{
	// Make sure the error output is hidden
	document.getElementById("errorMessageContainer").style.display="none";
	
	// Synthesizing the bloody optimization observations
	var theForm = document.getElementById(formId);
	var fieldMarkers = ["Date","Height","MSC","FEm","NDF","INDF","RawProtein"];
	var numberOfOptimizationInfoRows = 4;
	for(var i=0;i<numberOfOptimizationInfoRows;i++)
	{
		var paramArray = [];
		var dateIsValid = true;
		var rowIsValid = true;
		var numberOfEssentialFieldsWithValue = 0;
		for(var j in fieldMarkers)
		{
			//console.log("observation" + fieldMarkers[j] + "_" + i);
			var currentField = theForm["observation" + fieldMarkers[j] + "_" + i];
			//console.log(currentField.value);
			if(j == 0 && (currentField.value == "" || !moment(currentField.value, "YYYY-MM-DD").isValid()))
			{
				dateIsValid = false;
				rowIsValid = false;
			}
			else if(j > 0 && j <= 2 && dateIsValid && $.isNumeric(currentField.value.replace(",",".")))
			{
				numberOfEssentialFieldsWithValue ++;
			}
			//console.log("Numeric? " + $.isNumeric(currentField.value));
			
			paramArray.push(currentField.value.replace(",","."));
			// Discard it to avoid pollution of query string
			//currentField.disabled = true;
		}
		
		if(numberOfEssentialFieldsWithValue == 0 && dateIsValid)
		{
			rowIsValid = false;
			alert("Feil i kalibreringsdata. Høyde og/eller MSC må være utfylt.");
			return;
		}
		
		// Add comma separated values from one row (if valid) to a field
		var optimizationInfoField = document.getElementById("optimizationInfo_" + i);
		
		if(rowIsValid)
		{
			optimizationInfoField.value = paramArray.join(",");
		}
		else
		{
			// Disable this field as well if no valid data
			optimizationInfoField.disabled = true;
		}
	}
	
	// Synthesize watering lines
	var wateringParams= "";
	var fieldSets = document.getElementById("waterings").getElementsByTagName("fieldset");
	for(var i=0;i<fieldSets.length;i++)
	{
		var fieldInputs = fieldSets[i].getElementsByTagName("input");
		var wateringDate, wateringAmount;
		for(var j=0;j<2;j++)
		{
			// Disable to reduce noise in query string
			fieldInputs[j].disabled = true;
			if(fieldInputs[j].name=="wateringDate")
			{
				wateringDate = fieldInputs[j].value; 
			}
			else if(fieldInputs[j].name=="wateringAmount")
			{
				wateringAmount = fieldInputs[j].value.replace(",","."); 
			}
		}
		wateringParams += "&watering=" + wateringDate + "," + wateringAmount;
	}
	
	var formStr = $("#" + formId).serialize() + wateringParams;
	var request = $.ajax({
        type:"GET",
        url: settings.vipslogicProtocol + "://" + settings.vipslogicServerName + "/rest/roughage/nutrition/runmodel/" + settings.vipsOrganizationId + "?" + formStr,
        statusCode:{
            200: renderNutritionModelResults,
            400: handleAjaxError,
            401: handleAjaxError,
            404: handleAjaxError,
            500: handleAjaxError
        }
     });
	
	// Re-enable optimization fields
	for(var i=0;i<numberOfOptimizationInfoRows;i++)
	{
		for(var j in fieldMarkers)
		{
			//console.log("observation" + fieldMarkers[j] + "_" + i);
			var currentField = theForm["observation" + fieldMarkers[j] + "_" + i];
			currentField.disabled = false;
		}
		var optimizationInfoField = document.getElementById("optimizationInfo_" + i);
		optimizationInfoField.disabled = false;
	}
	// Re-enable watering fields
	var inputs = document.getElementById("waterings").getElementsByTagName("input");
	for(var i=0;i<inputs.length;i++)
	{
		inputs[i].disabled = false;
	}
}

/**
 * Callback function for runNutritionModel
 */
var renderNutritionModelResults = function(data,textStatus, jqXHR)
{
	stopAnimateHourGlass("modelRunHourGlass");
	Highcharts.setOptions({
		global: {
			 
	            getTimezoneOffset: function (timestamp) {
		                var zone = 'Europe/Oslo';
		                momentObj = moment.tz(timestamp, zone);
		                //console.log(momentObj);
		                timezoneOffset = -momentObj.utcOffset();
		                return timezoneOffset;
		            }

		}
	});
	
	
	data.sort(compareForecastResults);
	//console.log(data);
	
	// CHARTS
	// Depending on forecasts.js (in forecast app)
	var customOptions = {
			backgroundColor: "#f0f0f0",
			exporting: {
		        chartOptions: { // specific options for the exported image
		            plotOptions: {
		                series: {
		                    dataLabels: {
		                        enabled: true
		                    }
		                }
		            }
		        },
		        fallbackToExportServer: false
			}
	};
	var normalValueKeySuffix = " (Normalverdi)";
	var charts = [
	              	{key:"ROUGHAGENU.Beregnet MSC", name:"MSC", legendTitle:"MSC", chartTitle: "Utvikling", containerId: "MSCchartContainer"},
	              	{key:"ROUGHAGENU.Avling", name:"kg tørrstoff / daa", legendTitle:"kg tørrstoff / daa", chartTitle: "Avling", containerId: "harvestChartContainer"},
	              	{key:"ROUGHAGENU.NDF", name:"prosent av tørrstoff", legendTitle:"prosent av tørrstoff", chartTitle: "NDF", containerId: "NDFChartContainer"},
	              	{key:"ROUGHAGENU.INDF", name:"prosent av tørrstoff", legendTitle:"prosent av tørrstoff", chartTitle: "INDF", containerId: "INDFChartContainer"},
	              	{key:"ROUGHAGENU.R&aring;protein", name:"prosent av tørrstoff", legendTitle:"prosent av tørrstoff", chartTitle: "Råprotein", containerId: "rawProteinChartContainer"},
	              	{key:"ROUGHAGENU.FEm", name:"FEm pr. kg tørrstoff", legendTitle:"FEm pr. kg tørrstoff", chartTitle: "FEm", containerId: "FEmChartContainer"}
	             ];
	for(var i in charts)
	{
		var lines = [{key: charts[i].key, type: "spline", color: "red", name: charts[i].name, showInLegend: false, yAxis: 0}];
		
		if(getJSON(data[0].keys).indexOf(charts[i].key + normalValueKeySuffix) >= 0)
		{
			lines.push({key: charts[i].key + normalValueKeySuffix, type: "spline", color: "green", name: charts[i].name + "_NORMAL", showInLegend: true, yAxis: 0})
		}
		var currentSeries = getHighChartsSeries(
        	data,
        	[
        		{"color": "#6600FF", "opposite": false, "abbr": " ", "title": charts[i].legendTitle}
        	],
        	lines
        );
		//console.log(currentSeries);
		renderForecastChart(charts[i].containerId, charts[i].chartTitle, [], currentSeries, customOptions);
	}
	
	
	
	// TABLE
	renderResultTable("resultTable",data);
	document.getElementById("resultTableContainer").style.display="block";
	
	// Run the grass drying model. Shipping data from nutrition model
	runGrassDryingModel(data);
};

/**
 * Load weather station info from VIPSLogic, render results in form
 */
function initWeatherStations()
{
		// Fetching information asynchronously from server 
	    var request = $.ajax({
	        type:"GET",
	        url: settings.vipslogicProtocol + "://" + settings.vipslogicServerName + "/rest/poi/organization/" + settings.vipsOrganizationId,
	        statusCode:{
	            200: function(data,textStatus, jqXHR){
	            	// Building result HTML
	            	var wsHTML=["<option value=\"-1\">-- " + gettext("Select weather station") + " --</option>"];
	            	data.sort(compareWeatherStations);
	            	for(var i in data)
	            	{
	            		var ws = data[i];
	            		wsHTML.push("<option value=\"" + ws["pointOfInterestId"] + "\">" + ws["name"] + "</option>");
	            	}
	            	document.getElementById("weatherStationId").innerHTML = wsHTML.join("");
	        	},
	            400: handleAjaxError,
	            401: handleAjaxError,
	            404: handleAjaxError,
	            500: handleAjaxError
	        }
	     });
}

/**
 *  Load normal data weather station info from VIPSLogic, render results in form
 */
function initNormalDataWeatherStations()
{
		// Fetching information asynchronously from server 
	    var request = $.ajax({
	        type:"GET",
	        url: settings.vipslogicProtocol + "://" + settings.vipslogicServerName + "/rest/roughage/normaldataweatherstations/",
	        statusCode:{
	            200: function(data,textStatus, jqXHR){
	            	// Building result HTML
	            	var wsHTML=["<option value=\"-1\">-- " + gettext("Don't use normaldata") + " --</option>"];
	            	data.sort(compareWeatherStations);
	            	for(var i in data)
	            	{
	            		var ws = data[i];
	            		wsHTML.push("<option value=\"" + ws["pointOfInterestId"] + "\">" + ws["name"] + "</option>");
	            	}
	            	document.getElementById("normalDataWeatherStationId").innerHTML = wsHTML.join("");
	        	},
	            400: handleAjaxError,
	            401: handleAjaxError,
	            404: handleAjaxError,
	            500: handleAjaxError
	        }
	     });
}



//Error handling when collecting forecast information from server fails
var handleAjaxError = function(jqXHR,textStatus,errorThrown){
    var message = jqXHR.responseJSON !== undefined ? jqXHR.responseJSON.message : jqXHR.responseText;
    document.getElementById("runningModelMessage").style.display="none";
    document.getElementById("errorMessage").innerHTML = "<h1>Error</h1><pre>" + message + "</pre>";
    document.getElementById("errorMessageContainer").style.display="block";
};

/**
 * Dynamic adding of form inputs for a watering event
 */
function addWateringLine()
{
	var fieldSets = document.getElementById("waterings").getElementsByTagName("fieldset");
	var maxFieldSetId = -1;
	for(var i=0;i<fieldSets.length;i++)
	{
		//console.log(fieldSets[i]);
		var fieldSetId = parseInt(fieldSets[i].id.split("_")[1]);
		maxFieldSetId = Math.max(fieldSetId,maxFieldSetId);
	}
	var wateringLineHtml = 
	['<fieldset id="wateringLine_' + (++maxFieldSetId) + '" class="form-inline">',
	 '	<div class="form-group">',
	 '		<input type="date" class="form-control" name="wateringDate"/>',
	 '	</div>',
	 '	<div class="form-group">',
	 '		<input type="number" class="form-control" name="wateringAmount" placeholder="' + gettext('Watering amount') + ' (mm)"/>',
	 '	</div>',
	 '	<div class="form-group">',
	 '		<button type="button" class="btn btn-danger" onclick="deleteWateringLine(' + maxFieldSetId + ');"><span class="fa fa-times"> </span></button>',
	 '	</div>',
	 '</fieldset>'].join("\n");
	var tempDiv = document.createElement("div");
	tempDiv.innerHTML = wateringLineHtml;
	//var newLine = document.getElementById("wateringsNewLine").firstChild;
	document.getElementById("waterings").appendChild(tempDiv.firstChild);
	// Add date picker if necessary
	if (!Modernizr.inputtypes.date) {
		$('input[type=date]').datepicker({ dateFormat: 'yy-mm-dd' });
	}
}

/**
 * Dynamic deleting of inputs for a watering event
 * @param lineId
 */
function deleteWateringLine(lineId)
{
	var lineToRemove = document.getElementById("wateringLine_" + lineId);
	lineToRemove.parentNode.removeChild(lineToRemove);
}

/**
 * Display all results in a table
 * @param tableId
 * @param data
 */
function renderResultTable(tableId, data)
{
	var theTable=document.getElementById(tableId);
	//console.log(data);
	var headingData = getJSON(data[0].keys);
	var headerRowHTML =   "<thead>"
	                    + "	<tr>"
	                    + "		<th>" + gettext("Date") + "</th>";
	for(var i=0;i<headingData.length;i++)
	{
		headerRowHTML +=  "		<th>" + headingData[i].split(".")[1] + "</th>";
	}
	headerRowHTML +=	  "	</tr>"
						+ "</thead>";
	
	var dataRowsHTML = "<tbody>";
	for(var i=0; i<data.length; i++)
	{
		dataRowsHTML += "<tr><td>" + moment.tz(data[i].validTimeStart, "Europe/Oslo").format("YYYY-MM-DD") + "</td>";
		var allValues = getJSON(data[i].allValues);
		for(var j=0;j<headingData.length;j++)
		{
			dataRowsHTML += "<td>" + allValues[headingData[j]] + "</td>";
		}
		dataRowsHTML += "</tr>";
	}
	dataRowsHTML += "</tbody>";
	theTable.innerHTML = headerRowHTML + dataRowsHTML;
}


/**
 * 
 * @deprecated
 */
function enableGrassDryingModelFields()
{
	document.getElementById("grassDryingModelFields").disabled=false;
}

/**
 * 
 * @deprecated
 */
function disableGrassDryingModelFields()
{
	document.getElementById("grassDryingModelFields").disabled=true;
}


/**
 * Show/hide grass drying model fields
 * @param theCheckBox
 */
function toggleGrassDryingModelFields(theCheckBox)
{
	if(theCheckBox.checked)
	{
		//enableGrassDryingModelFields();
		document.getElementById("grassDryingModelFields").style.display="block";
	}
	else
	{
		//disableGrassDryingModelFields();
		document.getElementById("grassDryingModelFields").style.display="none";
	}
}

var grassDryingModelInputFieldsFirstHarvest = [
                              "harvestWidthFirstHarvest","yieldFirstHarvest",
		                      "conditioningFirstHarvest","stringWidthFirstHarvest","initialDryMatterFractionFirstHarvest"
		                      ];
var grassDryingModelInputFieldsSecondHarvest = [
		                      "harvestWidthSecondHarvest","yieldSecondHarvest",
		                      "conditioningSecondHarvest","stringWidthSecondHarvest","initialDryMatterFractionSecondHarvest"
		                     ];
var grassDryingModelOutputFields = [
							"harvestWidth","yield",
							"conditioning","stringWidth","initialDryMatterFraction"
                             ];

/**
 * Prepare the running of the grass drying model
 * @param nutritionModelResults
 */
function runGrassDryingModel(nutritionModelResults)
{
	var theForm = document.getElementById("grassDrying");
	if(!theForm["isRunGrassDryingModel"].checked)
	{
		return;
	}
	
	//console.log("Her må det kjøres tørkefartmodell!");
	var nutritionForm = document.getElementById("roughageNutrition");
	
	// Run model for first harvest
	runGrassDryingModelForHarvest(nutritionModelResults, 1);
	
	// Run model for second harvest as well, if date set
	if(nutritionForm["secondHarvest"].value != "")
	{
		runGrassDryingModelForHarvest(nutritionModelResults, 2);
	}
	
}

/**
 * Run grass drying model for a specific harvest
 * @param nutritionModelResults
 * @param seasonHarvestNumber
 */
function runGrassDryingModelForHarvest(nutritionModelResults, seasonHarvestNumber)
{
	var theForm = document.getElementById("grassDrying");
	var nutritionForm = document.getElementById("roughageNutrition");
	var weatherStationId = nutritionForm["weatherStationId"].options[nutritionForm["weatherStationId"].selectedIndex].value;
	var timeZone = "Europe/Oslo";
	var formStr = "seasonHarvestNumber=" + seasonHarvestNumber;
	var inputFields = (seasonHarvestNumber == 1 ? grassDryingModelInputFieldsFirstHarvest : grassDryingModelInputFieldsSecondHarvest);
	for(var i=0; i < inputFields.length;i++)
	{
		formStr += "&" + grassDryingModelOutputFields[i] + "=" + theForm[inputFields[i]].value.replace(",",".");
	}
	// Create a timestamp from timeOfHarvest
	var timeOfHarvest =  moment.tz(nutritionForm[(seasonHarvestNumber == 1 ? "first" : "second" ) + "Harvest"].value + " " + theForm["timeOf" + (seasonHarvestNumber == 1 ? "First" : "Second" ) + "Harvest"].value + ":00", timeZone).format();
	formStr += "&timeOfHarvest=" + encodeURIComponent(timeOfHarvest)
				+"&timeZone=" + timeZone + "&weatherStationId=" + weatherStationId;
	//console.log(timeOfHarvest);
	
	var request = $.ajax({
        type:"GET",
        url: settings.vipslogicProtocol + "://" + settings.vipslogicServerName + "/rest/roughage/grassdrying/runmodel/" + settings.vipsOrganizationId + "?" + formStr,
        statusCode:{
            200: (seasonHarvestNumber == 1 ? renderGrassDryingModelResultsFirstHarvest : renderGrassDryingModelResultsSecondHarvest),
            400: handleAjaxError,
            401: handleAjaxError,
            404: handleAjaxError,
            500: handleAjaxError
        }
     });
}

/**
 * Display grass drying model results from first harvest
 */
var renderGrassDryingModelResultsFirstHarvest = function(data,textStatus, jqXHR)
{
	renderGrassDryingModelResults(data,textStatus, jqXHR, 1);
};

/**
 * Display grass drying model results from second harvest
 */
var renderGrassDryingModelResultsSecondHarvest = function(data,textStatus, jqXHR)
{
	renderGrassDryingModelResults(data,textStatus, jqXHR, 2);
};

/**
 * Display grass drying model results
 */
var renderGrassDryingModelResults = function(data,textStatus, jqXHR, seasonHarvestNumber)
{
	//console.log("renderGrassDryingModelResults");
	var customOptions = {backgroundColor: "#f0f0f0"};
	
	var chartInfo = {key:"GRASSDRYMO.DRY_MATTER", name:"% tørrstoff", legendTitle:"% tørrstoff", chartTitle: "Tørkefartmodellen for " + (seasonHarvestNumber == 1 ? "første" : "andre") + "slått", containerId: "dryMatter" + (seasonHarvestNumber == 1 ? "First" : "Second") + "HarvestChartContainer"}

	var lines = [{key: chartInfo.key, type: "spline", color: "red", name: chartInfo.name, showInLegend: false, yAxis: 0}];
	
	var currentSeries = getHighChartsSeries(
		data,
		[
			{"color": "#6600FF", "opposite": false, "abbr": " ", "title": chartInfo.legendTitle}
		],
		lines
	);
	//console.log(currentSeries);
	renderForecastChart(chartInfo.containerId, chartInfo.chartTitle, [], currentSeries, customOptions);
};

/*
 * Hour glass animation for <span/> element
 * Depending on FontAwesome
 */

var hourGlassClasses = ["fa fa-hourglass","fa fa-hourglass-start","fa fa-hourglass-half","fa fa-hourglass-end","fa fa-hourglass-o"];
var hourGlassClassCounter = 0;
var hourGlassIntervalId = null;

var startAnimateHourGlass = function(hourGlassElId)
{
	var elm = document.getElementById(hourGlassElId);
	hourGlassIntervalId = setInterval(function(){ animateHourGlass(elm) }, 400);
}

var animateHourGlass = function(hourGlassEl){
	hourGlassEl.className=hourGlassClasses[hourGlassClassCounter++ % 5];
};

var stopAnimateHourGlass = function(hourGlassElId)
{
	clearInterval(hourGlassIntervalId);
	var elm = document.getElementById(hourGlassElId);
	hourGlassClassCounter = 0;
	elm.className="";
}



